(function() {
  var async, autoSpec, autospec, checkPaths, coffee, displayNotification, exec, fs, gatherFiles, handleError, intialTestComplete, libDir, main, path, prodCoffeeFiles, prodDeployDir, prodSrcDir, rootDir, runTests, spec, specCoffeeFiles, specFileDir, stalker, test, unWatchCoffeeFile, util, walk, watchCoffeeFile, writeSpecFile;
  fs = require('fs');
  exec = require('child_process').exec;
  util = require('util');
  stalker = require('stalker');
  path = require('path');
  async = require('async');
  libDir = path.join(path.dirname(fs.realpathSync(__filename)), '../lib');
  rootDir = process.cwd();
  prodSrcDir = "" + rootDir + "/src";
  specFileDir = "" + rootDir + "/spec";
  prodDeployDir = "" + rootDir + "/lib";
  prodCoffeeFiles = [];
  specCoffeeFiles = [];
  intialTestComplete = false;
  autoSpec = false;
  checkPaths = function(callback) {
    path.exists(prodDeployDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + prodDeployDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
    path.exists(specFileDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + specFileDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
    return path.exists(prodSrcDir, function(exists) {
      if (!exists) {
        return exec("mkdir " + prodSrcDir, function(err, stdout, stderr) {
          if (err) {
            return callback(err);
          }
        });
      }
    });
  };
  exports.run = function() {
    var args, command, commands, key, optimist, value, _ref, _ref2;
    checkPaths(function(err) {
      if (err) {
        handleError(err);
      }
    });
    optimist = require('optimist');
    args = optimist.argv;
    optimist.alias('test', 't');
    optimist.alias('spec', 's');
    optimist.alias('autospec', 'a');
    optimist.usage("Kitkat - Kontinuous Integrated Testing Koffee Application Template\n\n default usage with no arguments monitors current project for changes and runs all spec files found against all coffee files found.", {
      'test': {
        description: 'runs all tests against your application js files in the spec\n\t\tpath unless overridden in the dependency config file\n',
        required: false
      },
      'spec': {
        description: 'generates spec files based on the supplied path/file, if none\n\t\tprovided then processes all coffee files in project\n',
        required: false
      },
      'autospec': {
        description: 'generates new spec files and monitors all files for changes,\n\t\tautomatically runs tests on startup and as files change\n',
        required: false
      }
    });
    commands = [];
    for (key in args) {
      value = args[key];
      if (key !== "$0" && key !== "_") {
        try {
          switch (key.toLowerCase()) {
            case "t":
            case "test":
            case "s":
            case "spec":
            case "a":
            case "autotest":
              commands.push(key);
              break;
            default:
              throw "Invaid argument";
          }
        } catch (ex) {
          util.log("" + ex + "\n");
          optimist.showHelp();
          return;
        }
      }
    }
    command = (_ref = commands[0]) != null ? _ref.toLowerCase() : void 0;
    switch (command) {
      case "t":
      case "test":
        util.log("Initiating tests.......");
        return test();
      case "s":
      case "spec":
        util.log("Generating spec files......");
        return spec((_ref2 = args.s) != null ? _ref2 : args.s = args.spec);
      case "a":
      case "autospec":
        util.log("Autospec and testing current project.....");
        return autospec();
      default:
        util.log("Monitoring and testing current project.....");
        return main();
    }
  };
  autospec = function() {
    autoSpec = true;
    return main();
  };
  coffee = function(options, file) {
    if (options == null) {
      options = "";
    }
    util.log("Compiling " + file);
    return exec("coffee " + options + " --compile " + file, function(err, stdout, stderr) {
      if (err) {
        return handleError(err);
      }
    });
  };
  displayNotification = function(message) {
    var options;
    if (message == null) {
      message = '';
    }
    return options = {
      title: 'CoffeeScript',
      image: "" + libDir + "/CoffeeScript.png"
    };
  };
  gatherFiles = function(filepath, extRegex, callback) {
    var file, filesFound;
    filesFound = [];
    file = path.normalize(filepath);
    return fs.stat(file, function(err, stats) {
      if (err) {
        file += extRegex.toString().replace(/\//g, "").replace(/\\/g, "");
        return fs.stat(file, function(err, stats) {
          if (err) {
            callback(err, null);
          }
          if (!err) {
            filesFound.push(file);
            return callback(null, filesFound);
          }
        });
      } else {
        if (stats.isDirectory()) {
          return walk(path.normalize(file + '/'), extRegex, function(err, results) {
            if (err) {
              callback(err, null);
            }
            if (!err) {
              filesFound = results;
              return callback(null, filesFound);
            }
          });
        } else {
          filesFound.push(file);
          return callback(null, filesFound);
        }
      }
    });
  };
  handleError = function(error) {
    util.log(error);
    return displayNotification(error);
  };
  main = function() {
    stalker.watch("" + prodSrcDir, watchCoffeeFile, unWatchCoffeeFile);
    stalker.watch("" + specFileDir, watchCoffeeFile, unWatchCoffeeFile);
    return test();
  };
  runTests = function(callback) {
    var message, options;
    message = "";
    options = {};
    return exec("" + libDir + "/../bin/expresso spec/*.coffee -I lib", function(err, stdout, stderr) {
      var output, _ref, _ref2;
      if (err) {
        message = "" + ((_ref = stderr.match(/Failures.*/)) != null ? _ref.toString() : void 0);
        message = message.replace(/\[\d*m/g, "");
        output = "\n\n" + stderr.toString();
        output = output.replace(/at .*\n/g, "");
        util.log(output);
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/error.png",
          msg: "" + message
        };
      } else {
        options = {
          title: 'Automated Testing',
          image: "" + libDir + "/success.png",
          msg: "100% - " + ((_ref2 = stderr.match(/\d* tests/)) != null ? _ref2.toString() : void 0) + " passed"
        };
      }
      exec("growlnotify --name 'kitkat' --title '" + options.title + "' --message '" + options.msg + "' --image " + options.image + " --sticky");
      return callback(null);
    });
  };
  spec = function(filepath) {
    return async.series({
      gather_files: function(callback) {
        if (filepath === true) {
          filepath = "/";
        }
        filepath = path.join(prodSrcDir, path.normalize(filepath));
        return gatherFiles(filepath, /\.coffee/, function(err, results) {
          prodCoffeeFiles = results;
          return callback(err, results);
        });
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        pending = prodCoffeeFiles.length;
        _results = [];
        for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
          file = prodCoffeeFiles[index];
          _results.push((function(file, index) {
            var specFile;
            specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
            fs.stat(specFile, function(err, stats) {
              var prodFile;
              if (err) {
                prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js");
                return writeSpecFile(specFile, prodFile, function(err, results) {
                  if (err) {
                    return callback(err, null);
                  }
                });
              }
            });
            if (!--pending) {
              return callback(null, null);
            }
          })(file, index));
        }
        return _results;
      }
    }, function(err, results) {
      if (err) {
        return handleError(err);
      }
    });
  };
  test = function() {
    return async.series({
      gather_files: function(callback) {
        return async.parallel({
          gather_specs: function(callback) {
            return gatherFiles(specFileDir, /\.spec\.coffee/, function(err, results) {
              specCoffeeFiles = results;
              return callback(err, results);
            });
          },
          gather_prod: function(callback) {
            return gatherFiles(prodSrcDir, /\.coffee/, function(err, results) {
              prodCoffeeFiles = results;
              return callback(err, results);
            });
          }
        }, function(err, results) {
          return callback(err, results);
        });
      },
      create_specs: function(callback) {
        var file, index, pending, _len, _results;
        if (autoSpec) {
          pending = prodCoffeeFiles.length;
          _results = [];
          for (index = 0, _len = prodCoffeeFiles.length; index < _len; index++) {
            file = prodCoffeeFiles[index];
            _results.push((function(file, index) {
              var specFile;
              specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee");
              fs.stat(specFile, function(err, stats) {
                var prodFile;
                if (err) {
                  prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js");
                  writeSpecFile(specFile, prodFile, function(err, results) {
                    if (err) {
                      return callback(err, null);
                    }
                  });
                  return specCoffeeFiles.push(specFile);
                }
              });
              if (!--pending) {
                return callback(null, null);
              }
            })(file, index));
          }
          return _results;
        } else {
          return callback(null, null);
        }
      },
      run_specs: function(callback) {
        return runTests(function(err) {
          intialTestComplete = true;
          return callback(err, null);
        });
      }
    }, function(err, results) {
      if (err) {
        return handleError(err);
      }
    });
  };
  unWatchCoffeeFile = function(err, file) {
    if (("" + file).match(/\.coffee/) != null) {
      util.log("" + file + " removed.");
      fs.unwatchFile("" + file);
      return runTests(function(err) {});
    }
  };
  walk = function(dir, extRegex, callback) {
    var pending, results;
    results = [];
    pending = 0;
    return fs.readdir(dir, function(err, list) {
      var file, _i, _len, _results;
      if (err) {
        callback(err, null);
      }
      pending = list.length;
      if (pending <= 0) {
        callback(null, null);
      }
      _results = [];
      for (_i = 0, _len = list.length; _i < _len; _i++) {
        file = list[_i];
        _results.push((function(file) {
          file = dir + file;
          return fs.stat(file, function(err, stat) {
            if (stat && stat.isDirectory()) {
              file += "/";
              if (("" + file).match(/node_modules/) != null) {
                if (!--pending) {
                  return callback(null, results);
                }
              } else {
                return walk(file, extRegex, function(err, res) {
                  results = results.concat(res);
                  if (!--pending) {
                    return callback(null, results);
                  }
                });
              }
            } else {
              if (file.match(extRegex)) {
                results.push(file);
              }
              if (!--pending) {
                return callback(null, results);
              }
            }
          });
        })(file));
      }
      return _results;
    });
  };
  watchCoffeeFile = function(err, file) {
    var ignoreFileSpecDir;
    ignoreFileSpecDir = false;
    if (!(("" + file).match(/\.spec\.coffee/) != null) && (("" + file).match(/\.coffee/) != null)) {
      if (autoSpec === true && intialTestComplete === true) {
        spec("/");
      }
    }
    if (("" + file).indexOf(specFileDir) !== -1 && !(("" + file).match(/\.spec\.coffee/) != null)) {
      ignoreFileSpecDir = true;
    }
    if ((("" + file).match(/\.coffee/) != null) && !ignoreFileSpecDir) {
      util.log("Watch added for " + file);
      fs.unwatchFile("" + file);
      return fs.watchFile("" + file, function(curr, prev) {
        if (+curr.mtime !== +prev.mtime) {
          util.log("Saw change in " + file);
          if (("" + file).match(/\.spec\.coffee/)) {
            return runTests(function(err) {});
          } else {
            return exec("coffee --output " + prodDeployDir + " --compile " + file, function(err, stdout, stderr) {
              return runTests(function(err) {});
            });
          }
        }
      });
    }
  };
  writeSpecFile = function(specFile, prodFile, callback) {
    var libFile, specContent;
    libFile = path.basename(prodFile, path.extname(prodFile));
    specContent = ["replace with libfile", "assert = require('assert')", "module.exports =", "\t\"You need to modify this with real test cases and thus it fails until this file is modified.\": ->", "\t\tassert.ok false, \"it will fail\""];
    specContent[0] = "" + (libFile.replace(/\-/, "_")) + " = require('" + libFile + "')";
    fs.writeFile(specFile, specContent.join('\n'), 'utf8', function(err) {
      if (err) {
        callback(err, null);
      }
      return util.log("Created specfile '" + specFile + "'");
    });
    return callback(null, null);
  };
}).call(this);
