fs	   = require 'fs'
{exec} = require 'child_process'
util   = require 'util'
stalker = require 'stalker'
path = require 'path'
async = require 'async'

libDir			= path.join(path.dirname(fs.realpathSync(__filename)), '../lib')
rootDir			= process.cwd()

prodSrcDir		= "#{rootDir}/src"		# assumed that code files could be within the project anywhere
specFileDir		= "#{rootDir}/spec"
prodDeployDir	= "#{rootDir}/lib"	# This is the location for compiled code so that tests may be run against them, each file is compliled here as well as a combine js

prodCoffeeFiles = []
specCoffeeFiles = []

intialTestComplete = false
autoSpec = false


# Verify paths exist, create if they do not, send back error if any issues occured.
checkPaths = (callback) ->
	path.exists prodDeployDir, (exists) ->
		if not exists
			exec "mkdir #{prodDeployDir}", (err, stdout, stderr) ->
				if err
					callback err
	path.exists specFileDir, (exists) ->
		if not exists
			exec "mkdir #{specFileDir}", (err, stdout, stderr) ->
				if err
					callback err
	path.exists prodSrcDir, (exists) ->
		if not exists
			exec "mkdir #{prodSrcDir}", (err, stdout, stderr) ->
				if err
					callback err

# Entry point to kitkat
exports.run = ->
	# Commandline processor
	checkPaths (err) ->
		if err
			handleError err
			return
		
	optimist = require 'optimist'
	args = optimist.argv
	optimist.alias 'test', 't'
	optimist.alias 'spec', 's'
	optimist.alias 'autospec', 'a'
	optimist.usage("Kitkat - Kontinuous Integrated Testing Koffee Application Template\n\n default usage with no arguments monitors current project for changes and runs all spec files found against all coffee files found.", {
		'test': {description: 'runs all tests against your application js files in the spec\n\t\tpath unless overridden in the dependency config file\n', required: false},
		'spec': {description: 'generates spec files based on the supplied path/file, if none\n\t\tprovided then processes all coffee files in project\n', required: false},
		'autospec': {description: 'generates new spec files and monitors all files for changes,\n\t\tautomatically runs tests on startup and as files change\n', required: false}
	})
	commands = []
	for key, value of args
		if key != "$0" and key != "_"
			try 
				switch key.toLowerCase()
					when "t", "test", "s", "spec", "a", "autotest"
						commands.push key
					else
						throw "Invaid argument"
			catch ex
				util.log "#{ex}\n"
				optimist.showHelp() # invalid engine specificed, you do not have to specify an engine to use expresso
				return

	command = commands[0]?.toLowerCase()

	switch command
		when "t", "test"
			util.log "Initiating tests......."
			test() # args.t ?= args.test
		when "s", "spec"
			util.log "Generating spec files......"
			spec args.s ?= args.spec
		when "a", "autospec"
			util.log "Autospec and testing current project....."
			autospec()
		else
			util.log "Monitoring and testing current project....."
			main()

#########################################
# Functions follow in alphabetical order
#########################################

autospec = ->
	autoSpec = true
	main()

coffee = (options = "", file) ->
	util.log "Compiling #{file}"
	exec "coffee #{options} --compile #{file}", (err, stdout, stderr) -> 
		handleError(err) if err

displayNotification = (message = '') -> 
	options = {
		title: 'CoffeeScript'
		image: "#{libDir}/CoffeeScript.png"
	}
	## CHange to is growlnotify by exec growl.notify message, options

gatherFiles = (filepath, extRegex, callback) ->
	filesFound = []
	file = path.normalize(filepath)
	fs.stat file, (err,stats) ->
		if err
			file += extRegex.toString().replace(/\//g,"").replace(/\\/g,"")
			fs.stat file, (err,stats) ->
				callback err, null if err 
				if not err
					filesFound.push file 
					callback null, filesFound
		else
			if stats.isDirectory()
				walk path.normalize(file + '/'), extRegex, (err,results) ->
					callback err, null if err
					if not err
						filesFound = results	
						callback null, filesFound
			else
				filesFound.push file
				callback null, filesFound

handleError = (error) -> 
	util.log error
	displayNotification error

main = ->
	stalker.watch "#{prodSrcDir}" , watchCoffeeFile, unWatchCoffeeFile
	stalker.watch "#{specFileDir}" , watchCoffeeFile, unWatchCoffeeFile
	test()

runTests = (callback) ->
	message = ""
	options = {}
	exec "#{libDir}/../bin/expresso spec/*.coffee -I lib", (err, stdout, stderr) ->
		if err
			message = "" + stderr.match(/Failures.*/)?.toString()		# Extract failure count
			message = message.replace(/\[\d*m/g, "") 					# stripping special formatting from output
			output = "\n\n" + stderr.toString()
			output = output.replace(/at .*\n/g, "")
			util.log output
			options = {
				title: 'Automated Testing'
				image: "#{libDir}/error.png"
				msg: "#{message}"
			}
		else
			options = {
				title: 'Automated Testing'
				image: "#{libDir}/success.png"
				msg: "100% - " + stderr.match(/\d* tests/)?.toString() + " passed"		# Extract Test count
			}
		exec "growlnotify --name 'kitkat' --title '#{options.title}' --message '#{options.msg}' --image #{options.image} --sticky"

## Rewrite to use growlnotify		growl.notify message, options
		callback null

spec = (filepath) ->
	async.series
		gather_files: (callback) ->
			filepath = "/" if (filepath==true)
			filepath = path.join(prodSrcDir, path.normalize(filepath))
			gatherFiles filepath, /\.coffee/, (err, results) ->
				prodCoffeeFiles = results
				callback err, results
		create_specs: (callback) ->
			pending = prodCoffeeFiles.length
			for file, index in prodCoffeeFiles then do (file, index) ->
				specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee")
				fs.stat specFile, (err,stats) ->
					if err # File does not exist
						prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js")
						writeSpecFile specFile, prodFile, (err, results) ->
							callback err, null if err
				callback null, null unless --pending
	, (err, results) ->
		handleError(err) if err
		

test = ->
	async.series
		gather_files: (callback) ->
			async.parallel
				gather_specs: (callback) ->
					gatherFiles specFileDir, /\.spec\.coffee/, (err, results) ->
						specCoffeeFiles = results
						callback err, results
				gather_prod: (callback) ->
					gatherFiles prodSrcDir, /\.coffee/, (err, results) ->
						prodCoffeeFiles = results
						callback err, results
			, (err, results) ->
				callback err, results

		create_specs: (callback) ->
			if autoSpec
				pending = prodCoffeeFiles.length
				for file, index in prodCoffeeFiles then do (file, index) ->
					specFile = path.join(specFileDir, path.basename(file, '.coffee') + ".spec.coffee")
					fs.stat specFile, (err,stats) ->
						if err # File does not exist
							prodFile = path.join(prodDeployDir, path.basename(file, '.coffee') + ".js")
							writeSpecFile specFile, prodFile, (err, results) ->
								callback err, null if err
							specCoffeeFiles.push specFile
					callback null, null unless --pending
			else 
				callback null, null

		run_specs: (callback) ->
			runTests (err) ->
				intialTestComplete = true
				callback err, null

	, (err, results) ->
		handleError(err) if err

unWatchCoffeeFile = (err, file) ->
	if "#{file}".match(/\.coffee/)?
		util.log "#{file} removed."
		fs.unwatchFile "#{file}"
		runTests (err) ->
			return

walk = (dir, extRegex, callback) ->
	results = []
	pending = 0
	fs.readdir dir, (err, list) ->
		callback err, null if err
		pending = list.length
		callback null, null if pending <=0
		for file in list then do(file) ->
			file = dir + file
			fs.stat file, (err, stat) ->
				if stat and stat.isDirectory()
					file += "/"
					if "#{file}".match(/node_modules/)?
						callback null,results unless --pending
					else
						walk file, extRegex, (err, res) ->
							results = results.concat(res)
							callback null, results unless --pending
				else
					if file.match(extRegex)
						results.push file
					callback null, results unless --pending

watchCoffeeFile = (err, file) ->
	ignoreFileSpecDir = false
	if not "#{file}".match(/\.spec\.coffee/)? and "#{file}".match(/\.coffee/)?
		spec "/" if (autoSpec==true and intialTestComplete==true)
	if "#{file}".indexOf(specFileDir) != -1 and not "#{file}".match(/\.spec\.coffee/)?
		ignoreFileSpecDir = true
	if "#{file}".match(/\.coffee/)? and not ignoreFileSpecDir
		util.log "Watch added for #{file}"
		fs.unwatchFile "#{file}"
		fs.watchFile "#{file}", (curr, prev) ->
			if +curr.mtime isnt +prev.mtime
				util.log "Saw change in #{file}"
				if "#{file}".match(/\.spec\.coffee/)
					runTests (err) ->
						return
				else
					exec "coffee --output #{prodDeployDir} --compile #{file}", (err,stdout, stderr) ->
						runTests (err) ->
							return

writeSpecFile = (specFile, prodFile, callback) ->
	libFile = path.basename(prodFile, path.extname(prodFile))
	specContent = ["replace with libfile", "assert = require('assert')", "module.exports =", "\t\"You need to modify this with real test cases and thus it fails until this file is modified.\": ->", "\t\tassert.ok false, \"it will fail\"" ]
	specContent[0] = "#{libFile.replace(/\-/,"_")} = require('#{libFile}')"

	fs.writeFile specFile, specContent.join('\n'), 'utf8', (err) ->
		callback err, null if err
		util.log "Created specfile '#{specFile}'"
	callback null, null	